const effect_in_use : array[1..4] of boolean = (false,false,false,false);
procedure ErrorExit(s:string);
begin
     set_text;
     writeln('An unexpected error happened: ',s);
     writeln;
     writeln('Please report to the author: sinsch@informatik.uni-frankfurt.de');
     halt;
end;
function LoadEffect(fileName : PChar; looping : integer) : word;
var
    instHandle : word;                  { Sound Device instrument handle }
    error : integer;
    f : fileHandle;
    smpLength : longint;                { sample length }
    smpBuf : ^byte;                     { sample loading buffer }
begin
    { open sound effect file: }
    error := fileOpen(fileName, fileOpenRead, @f);
    if error <> OK then
        midasError(error);

    { get file length: }
    error := fileGetSize(f, @smpLength);
    if error <> OK then
        midasError(error);

    { check that sample length is not too long: }
    if smpLength > SMPMAX then
        midasError(errInvalidInst);

    { allocate memory for sample loading buffer: }
    error := memAlloc(smpLength, @smpBuf);
    if error <> OK then
        midasError(error);

    { load sample: }
    error := fileRead(f, smpBuf, smpLength);
    if error <> OK then
        midasError(error);

    { close sample file: }
    error := fileClose(f);
    if error <> OK then
        midasError(error);

    { Add sample to Sound Device list and get instrument handle to
      instHandle: }
    if looping = 1 then
    begin
        error := SD^.AddInstrument(smpBuf, smp8bit, smpLength, 0, smpLength,
            64, 1, 1, @instHandle);
    end
    else
    begin
        error := SD^.AddInstrument(smpBuf, smp8bit, smpLength, 0, 0, 64, 0,
            1, @instHandle);
    end;

    if error <> OK then
        midasError(error);

    { deallocate sample allocation buffer: }
    error := memFree(smpBuf);
    if error <> OK then
        midasError(error);

    { return instrument handle: }
    LoadEffect := instHandle;
end;
procedure PlayEffect(instHandle : word; rate : longint; volume : word;
    panning : integer);
var
    error : integer;
begin
    { set effect instrument to current effect channel: }
    error := SD^.SetInstrument(fxChannel, instHandle);
    if error <> OK then
        midasError(error);

    { set effect volume: }
    error := SD^.SetVolume(fxChannel, volume);
    if error <> OK then
        midasError(error);

    { set effect panning: }
    error := SD^.SetPanning(fxChannel, panning);
    if error <> OK then
        midasError(error);

    { start playing effect: }
    error := SD^.PlaySound(fxChannel, rate);
    if error <> OK then
        midasError(error);

    fxChannel := fxChannel + 1;         { channel for next effect }
    if fxChannel >= FXCHANNELS then
        fxChannel := 0;
end;
procedure Init_Music;
var str : array[0..256] of char;
begin
    { Check that the configuration file exists: }
    error := fileExists('GIANA.CFG', @isConfig);
    if error <> OK then
        midasError(error);
    if isConfig <> 1 then
    begin
         set_text;
        WriteLn('Configuration file not found - run SETUP');
        Halt;
    end;
    midasSetDefaults;                   { set MIDAS defaults }
    midasLoadConfig('GIANA.CFG');       { load configuration }
    midasInit;                          { initialize MIDAS Sound System }
    SD := midasSD;                      { copy Sound Device pointer }
    midasOpenChannels(FXCHANNELS + MAXMUSCHANNELS);
    error := SD^.GetAmplification(@defAmplify);
    if error <> OK then
        midasError(error);
    amplification := defAmplify;
    fxChannel := 0;
end;
procedure Midas_Sync;
begin
     sync_display;
end;
function toASCIIZ(dest : PChar; str : string) : PChar;
var
    spos, slen : integer;
    i : integer;

begin
    spos := 0;                          { string position = 0 }
    slen := ord(str[0]);                { string length }

    { copy string to ASCIIZ conversion buffer: }
    while spos < slen do
    begin
        dest[spos] := str[spos+1];
        spos := spos + 1;
    end;

    dest[spos] := chr(0);               { put terminating 0 to end of string }

    toASCIIZ := dest;
end;

procedure Play_Music(s:string);
begin
     Init_Music;
     toASCIIZ(ASCIIstr, s);
     module := midasLoadModule(ASCIIstr, @mpMOD, NIL);
     midasPlayModule(module, 0);
end;
procedure FreeEffect(instHandle : word);
var
    error : integer;
begin
    { remove instrument from Sound Device list: }
    error := SD^.RemInstrument(instHandle);
    if error <> OK then
        midasError(error);
end;
procedure Free_Music;
begin
     if effect_in_use[1] then begin FreeEffect(effect1); effect_in_use[1] := false; end;
     if effect_in_use[2] then begin FreeEffect(effect2); effect_in_use[2] := false; end;
     if effect_in_use[3] then begin FreeEffect(effect3); effect_in_use[3] := false; end;
     if effect_in_use[4] then begin FreeEffect(effect4); effect_in_use[4] := false; end;
     midasStopModule(module);            { stop playing }
     midasFreeModule(module);            { deallocate module }
     midasclose;
end;
procedure detect_midasconfig;
var f:file of Byte;
begin
     Assign(f,'GIANA.CFG');
     {$i-}
     Reset(f);
     {$i+}
     if ioresult <> 0 then
     begin
          WriteLn('Configuration file not found - run SETUP',#7);
          halt;
     end;
     close(f);
end;